@@ -12,6 +12,14 @@ const sidebars: SidebarsConfig = {
         'getting_started/installation',
       ],
     },
+    {
+      type: 'category',
+      label: 'Tutorials',
+      collapsed: false,
+      items: [
+        'tutorials/live_updates',
+      ],
+    },
     {
       type: 'category',
       label: 'CocoIndex Core',
@@ -23,7 +31,6 @@ const sidebars: SidebarsConfig = {
         'core/settings',
         'core/flow_methods',
         'core/cli',
-        'core/custom_function',
       ],
     },
     {
@@ -36,6 +43,15 @@ const sidebars: SidebarsConfig = {
         'ops/targets',
       ],
     },
+    {
+      type: 'category',
+      label: 'Custom Operations',
+      collapsed: false,
+      items: [
+        'custom_ops/custom_functions',
+        'custom_ops/custom_targets',
+      ],
+    },
     {
       type: 'category',
       label: 'AI Support',
