@@ -66,14 +66,19 @@ class SentenceTransformerEmbed(op.FunctionSpec):
     args: dict[str, Any] | None = None
 
 
-@op.executor_class(gpu=True, cache=True, behavior_version=1)
+@op.executor_class(
+    gpu=True,
+    cache=True,
+    behavior_version=1,
+    arg_relationship=(op.ArgRelationship.EMBEDDING_ORIGIN_TEXT, "text"),
+)
 class SentenceTransformerEmbedExecutor:
     """Executor for SentenceTransformerEmbed."""
 
     spec: SentenceTransformerEmbed
     _model: Any | None = None
 
-    def analyze(self, text: Any) -> type:
+    def analyze(self, _text: Any) -> type:
         try:
             # Only import sentence_transformers locally when it's needed, as its import is very slow.
             import sentence_transformers  # pylint: disable=import-outside-toplevel
@@ -88,11 +93,7 @@ class SentenceTransformerEmbedExecutor:
         args = self.spec.args or {}
         self._model = sentence_transformers.SentenceTransformer(self.spec.model, **args)
         dim = self._model.get_sentence_embedding_dimension()
-        result: type = Annotated[
-            Vector[np.float32, Literal[dim]],  # type: ignore
-            TypeAttr("cocoindex.io/vector_origin_text", text.analyzed_value),
-        ]
-        return result
+        return Vector[np.float32, Literal[dim]]  # type: ignore
 
     def __call__(self, text: str) -> NDArray[np.float32]:
         assert self._model is not None
