@@ -8,6 +8,7 @@ class LlmApiType(Enum):
     OPENAI = "OpenAi"
     OLLAMA = "Ollama"
     GEMINI = "Gemini"
+    VERTEX_AI = "VertexAi"
     ANTHROPIC = "Anthropic"
     LITE_LLM = "LiteLlm"
     OPEN_ROUTER = "OpenRouter"
@@ -15,6 +16,16 @@ class LlmApiType(Enum):
     VLLM = "Vllm"
 
 
+@dataclass
+class VertexAiConfig:
+    """A specification for a Vertex AI LLM."""
+
+    kind = "VertexAi"
+
+    project: str
+    region: str | None = None
+
+
 @dataclass
 class LlmSpec:
     """A specification for a LLM."""
@@ -22,3 +33,4 @@ class LlmSpec:
     api_type: LlmApiType
     model: str
     address: str | None = None
+    api_config: VertexAiConfig | None = None
