@@ -1,6 +1,9 @@
 use crate::{
-    lib_context::{FlowExecutionContext, LibSetupContext, get_auth_registry},
-    ops::{get_optional_executor_factory, interface::ExportTargetFactory},
+    lib_context::{FlowContext, FlowExecutionContext, LibSetupContext},
+    ops::{
+        get_optional_executor_factory,
+        interface::{ExportTargetFactory, FlowInstanceContext},
+    },
     prelude::*,
 };
 
@@ -252,6 +255,7 @@ fn group_resource_states<'a>(
 pub async fn check_flow_setup_status(
     desired_state: Option<&FlowSetupState<DesiredMode>>,
     existing_state: Option<&FlowSetupState<ExistingMode>>,
+    flow_instance_ctx: &Arc<FlowInstanceContext>,
 ) -> Result<FlowSetupStatus> {
     let metadata_change = diff_state(
         existing_state.map(|e| &e.metadata),
@@ -331,7 +335,7 @@ pub async fn check_flow_setup_status(
                         &resource_id.key,
                         target_state,
                         existing_without_setup_by_user,
-                        get_auth_registry(),
+                        flow_instance_ctx.clone(),
                     )
                     .await?,
             )
@@ -387,7 +391,12 @@ async fn maybe_update_resource_setup<
                 });
                 writeln!(write, "{}:", resource.description)?;
                 for change in setup_status.describe_changes() {
-                    writeln!(write, "  - {change}")?;
+                    match change {
+                        setup::ChangeDescription::Action(action) => {
+                            writeln!(write, "  - {action}")?;
+                        }
+                        setup::ChangeDescription::Note(_) => {}
+                    }
                 }
             }
         }
@@ -402,7 +411,7 @@ async fn maybe_update_resource_setup<
 
 async fn apply_changes_for_flow(
     write: &mut (dyn std::io::Write + Send),
-    flow_name: &str,
+    flow_ctx: &FlowContext,
     flow_status: &FlowSetupStatus,
     existing_setup_state: &mut Option<setup::FlowSetupState<setup::ExistingMode>>,
     pool: &PgPool,
@@ -416,7 +425,7 @@ async fn apply_changes_for_flow(
         ObjectStatus::Existing => "Updating resources for ",
         _ => bail!("invalid flow status"),
     };
-    write!(write, "\n{verb} flow {flow_name}:\n")?;
+    write!(write, "\n{verb} flow {}:\n", flow_ctx.flow_name())?;
 
     let mut update_info =
         HashMap::<db_metadata::ResourceTypeKey, db_metadata::StateUpdateInfo>::new();
@@ -466,7 +475,7 @@ async fn apply_changes_for_flow(
     }
 
     let new_version_id = db_metadata::stage_changes_for_flow(
-        flow_name,
+        flow_ctx.flow_name(),
         flow_status.seen_flow_metadata_version,
         &update_info,
         pool,
@@ -508,7 +517,7 @@ async fn apply_changes_for_flow(
                                 setup_status: s.setup_status.as_ref(),
                             })
                             .collect(),
-                        get_auth_registry(),
+                        flow_ctx.flow.flow_instance_ctx.clone(),
                     )
                     .await?;
                 Ok(())
@@ -519,7 +528,7 @@ async fn apply_changes_for_flow(
 
     let is_deletion = status == ObjectStatus::Deleted;
     db_metadata::commit_changes_for_flow(
-        flow_name,
+        flow_ctx.flow_name(),
         new_version_id,
         &update_info,
         is_deletion,
@@ -571,7 +580,7 @@ async fn apply_changes_for_flow(
         });
     }
 
-    writeln!(write, "Done for flow {flow_name}")?;
+    writeln!(write, "Done for flow {}", flow_ctx.flow_name())?;
     Ok(())
 }
 
@@ -613,7 +622,7 @@ pub struct SetupChangeBundle {
 impl SetupChangeBundle {
     async fn get_flow_setup_status<'a>(
         setup_ctx: &LibSetupContext,
-        flow_name: &str,
+        flow_ctx: &'a FlowContext,
         flow_exec_ctx: &'a FlowExecutionContext,
         action: &FlowSetupChangeAction,
         buffer: &'a mut Option<FlowSetupStatus>,
@@ -621,8 +630,11 @@ impl SetupChangeBundle {
         let result = match action {
             FlowSetupChangeAction::Setup => &flow_exec_ctx.setup_status,
             FlowSetupChangeAction::Drop => {
-                let existing_state = setup_ctx.all_setup_states.flows.get(flow_name);
-                buffer.insert(check_flow_setup_status(None, existing_state).await?)
+                let existing_state = setup_ctx.all_setup_states.flows.get(flow_ctx.flow_name());
+                buffer.insert(
+                    check_flow_setup_status(None, existing_state, &flow_ctx.flow.flow_instance_ctx)
+                        .await?,
+                )
             }
         };
         Ok(result)
@@ -657,7 +669,7 @@ impl SetupChangeBundle {
             let mut setup_status_buffer = None;
             let setup_status = Self::get_flow_setup_status(
                 setup_ctx,
-                flow_name,
+                &flow_ctx,
                 &flow_exec_ctx,
                 &self.action,
                 &mut setup_status_buffer,
@@ -709,7 +721,7 @@ impl SetupChangeBundle {
             let mut setup_status_buffer = None;
             let setup_status = Self::get_flow_setup_status(
                 setup_ctx,
-                flow_name,
+                &flow_ctx,
                 &flow_exec_ctx,
                 &self.action,
                 &mut setup_status_buffer,
@@ -722,7 +734,7 @@ impl SetupChangeBundle {
             let mut flow_states = setup_ctx.all_setup_states.flows.remove(flow_name);
             apply_changes_for_flow(
                 write,
-                flow_name,
+                &flow_ctx,
                 setup_status,
                 &mut flow_states,
                 &persistence_ctx.builtin_db_pool,
